/*2.semestralni projekt do predmetu Zaklady pocitacove grafiky

Autor: Martin Kot
Studijni cislo: L97041
Nazev programu: Viditelnost

Popis programu: Program slouzi k zobrazeni konvexnich rovinnych mnohouhe-
lniku v prostoru do roviny 0xy. K urceni viditelnych casti mnohouhelniku
jsou implementovany 2 algoritmy:
1. spociva v nekolikanasobnem pouziti algoritmu z-buffer. (Pro nedostatek
pameti je pouzit buffer jen na jeden radek a algoritmus je pouzit tolikrat,
kolik je radku.)
2. algoritmus nema temer zadne naroky na pamet, je vsak pomalejsi. Postupne
je pro kazdy pixel urcena jeho barva tak, ze se zjisti z-souradnice mnoho-
uhelniku, ktere by se na neho mely zobrazit. Barvou mnohouhelniku s nejve-
tsi hodnotou z v danem pixelu je tento vykreslen.
Program umoznuje zobrazit cas potrebny pro vypocet a vykresleni.
Mnohouhelniky jsou zadavany pomoci textoveho souboru. V nem je kazdy mnoho-
uhelnik ulozen na n+1 radcich (n pocet vrcholu). Na prvnim radku je pocet
vrcholu a na dalsich n radcich jsou vzdy vsechny 3 souradnice (x,y,z)
vrcholu. Vrcholy na sousednich radcich musi mit spolecnou hranu.Na radku pod
souradnicemi posledniho vrcholu nasleduje jiz pocet vrcholu dalsiho utvaru*/


#include <stdio.h>
#include <conio.h>
#include <graphics.h>
#include <stdlib.h>
#include <math.h>
#include <alloc.h>
#include <time.h>

#define BODU_PLOCHY 3
#define TRUE 1
#define FALSE 0
#define BGI ""
#define RETEZEC 512
#define MAX_MNOH 15
#define OKRAJE 1.1

const float PI=M_PI;
const float odchylka=0.0001;
typedef enum {FORMAT_SOUB,MALO_SOUR,NEDEF_RNA,PROT_HRAN,NENI_VIDET,DOBRE} chyby;

typedef struct
{
  float x;
  float y;
  float z;
} bod3d;

typedef struct
{
  bod3d A;
  bod3d u;
  bod3d v;
  bod3d n;
  float d;
} rovina;

typedef struct{
  float a;
  float b;
  float c;
} polorovina;

typedef struct{
  int pocet_bodu;
  int max;
  int mix;
  int may;
  int miy;
  int miz;
  bod3d *body;
  rovina plocha;
  polorovina *pol;
  int barva;
}mnohouhelnik;

typedef struct{
  float sour_z;
  int barva;
} pol_buf;

int patriPlose(mnohouhelnik mnoh,bod3d X){
  if(X.x<mnoh.mix||X.x>mnoh.max)
    return FALSE;
  for(int i=0;i<mnoh.pocet_bodu;i++){
    if( (mnoh.pol[i].a*X.x + mnoh.pol[i].b*X.y + mnoh.pol[i].c) < 0)
      return FALSE;
  }
  return TRUE;
}

int vyplnBuffer(pol_buf *buffer,mnohouhelnik mnoh,float y,float Ox,float krok){
  int hrana1=-1;
  int hrana2=-1;
  float x1,x2,x3,z,pom;
  int i,index;
  for(i=0;i<mnoh.pocet_bodu;i++){
    if((mnoh.body[i].y<=y&&mnoh.body[(i+1)%mnoh.pocet_bodu].y>=y)
       ||(mnoh.body[i].y>=y&&mnoh.body[(i+1)%mnoh.pocet_bodu].y<=y)){
      hrana1=i;
      break;
    }
  }
  i++;
  for(;i<mnoh.pocet_bodu;i++){
    if((mnoh.body[i].y<=y&&mnoh.body[(i+1)%mnoh.pocet_bodu].y>=y)
       ||(mnoh.body[i].y>=y&&mnoh.body[(i+1)%mnoh.pocet_bodu].y<=y)){
      hrana2=i;
      break;
    }
  }
  if(hrana1==-1||hrana2==-1)
    return FALSE;
  if(mnoh.pol[hrana1].a<odchylka&&mnoh.pol[hrana1].a>-odchylka){
    x1=mnoh.body[hrana1].x;
    x2=mnoh.body[(hrana1+1)%mnoh.pocet_bodu].x;
  }
  else
    if(mnoh.pol[hrana2].a<odchylka&&mnoh.pol[hrana2].a>-odchylka){
      x1=mnoh.body[hrana2].x;
      x2=mnoh.body[(hrana2+1)%mnoh.pocet_bodu].x;
     }
     else{
       x1=(-mnoh.pol[hrana1].c-(mnoh.pol[hrana1].b*y))/mnoh.pol[hrana1].a;
       x2=(-mnoh.pol[hrana2].c-(mnoh.pol[hrana2].b*y))/mnoh.pol[hrana2].a;
     }
  if(x2<x1){
    x3=x2;
    x2=x1;
    x1=x3;
  }
  index=(x1-Ox)/krok;
  pom=(mnoh.plocha.n.y*y)+mnoh.plocha.d;
  for(x3=x1;x3<=x2;x3+=krok)
  {
    z=((-mnoh.plocha.n.x*x3)-pom)/mnoh.plocha.n.z;
    if(z>buffer[index].sour_z){
      buffer[index].sour_z=z;
      buffer[index].barva=mnoh.barva;
    }
    index++;
  }
  return TRUE;
}


bod3d operator*(bod3d v1, bod3d v2)
{
  bod3d vysl;
  vysl.x=v1.y*v2.z-v1.z*v2.y;
  vysl.y=v1.z*v2.x-v1.x*v2.z;
  vysl.z=v1.x*v2.y-v1.y*v2.x;
  return vysl;
}

bod3d operator-(bod3d v1, bod3d v2)
{
  bod3d vysl;
  vysl.x=v1.x-v2.x;
  vysl.y=v1.y-v2.y;
  vysl.z=v1.z-v2.z;
  return vysl;
}

rovina bodyRna(bod3d A, bod3d B, bod3d C)
{
  rovina r;
  r.A=A;
  r.u=(B-A);
  r.v=(C-A);
  r.n=(r.u*r.v);
  r.d=-((r.n.x*A.x)+(r.n.y*A.y)+(r.n.z*A.z));
  return r;
}


int bodyPolorna(bod3d A, bod3d B, bod3d C, polorovina *pol)
{
  polorovina r;
  float vysl;

  r.a=B.y-A.y;
  r.b=A.x-B.x;
  r.c=-r.a*A.x-r.b*A.y;
  vysl=r.a*C.x + r.b*C.y + r.c;
  if(vysl<odchylka&&vysl>-odchylka)
    return FALSE;
  if(r.a*C.x + r.b*C.y + r.c<0){
    r.a=-r.a;
    r.b=-r.b;
    r.c=-r.c;
  }
  *pol=r;
  return TRUE;
}

float skalSouc(bod3d v1, bod3d v2){
  return v1.x*v2.x+v1.y*v2.y+v1.z*v2.z;
}

float norma(bod3d vektor){
  return sqrt(vektor.x*vektor.x+vektor.y*vektor.y+vektor.z*vektor.z);
}


float uhelVekt(bod3d A,bod3d B,bod3d C){
  bod3d u,v;
  u=B-A;
  v=B-C;
  return acos(skalSouc(u,v)/(norma(u)*norma(v)));
}



int kontrola(bod3d *body, int pocet){
  float celkuhel=0;
  float uhel;
  for(int i=0;i<pocet;i++)
    celkuhel+=uhelVekt(body[i],body[(i+1)%pocet],body[(i+2)%pocet]);
  uhel=PI*(pocet-2);
  if(celkuhel>uhel-odchylka && celkuhel<uhel+odchylka)
    return TRUE;
  else
    return FALSE;
}


int utvorMnohouhelnik(bod3d *body, int pocet,mnohouhelnik *mn){
  mnohouhelnik vysledek;
  int  i,k;
  bod3d *nove;
  if((vysledek.pol=(polorovina*)calloc(pocet,sizeof(polorovina)))==NULL){
    fprintf(stderr,"Malo pameti pro beh programu!\n");
    exit(0);
  }
  vysledek.max=body[0].x;
  vysledek.mix=body[0].x;
  vysledek.may=body[0].y;
  vysledek.miy=body[0].y;
  vysledek.miz=body[0].z;

  vysledek.pocet_bodu=pocet;

  for(i=0;i<pocet;i++){
    k=2;
    while(bodyPolorna(body[i],body[(i+1)%pocet],body[(i+k)%pocet],&(vysledek.pol[i]))==FALSE){
      k++;
      if((i+k)%pocet==i)
	return FALSE;
    }
    if(body[i].x>vysledek.max)
      vysledek.max=body[i].x;
    if(body[i].x<vysledek.mix)
      vysledek.mix=body[i].x;
    if(body[i].y>vysledek.may)
      vysledek.may=body[i].y;
    if(body[i].y<vysledek.miy)
      vysledek.miy=body[i].y;
    if(body[i].z<vysledek.miz)
      vysledek.miz=body[i].z;
  }
  i=2;
  do{
    vysledek.plocha=bodyRna(body[0],body[1],body[i]);
    i++;
  }while(vysledek.plocha.n.x<odchylka&&vysledek.plocha.n.x>-odchylka
     &&vysledek.plocha.n.y<odchylka&&vysledek.plocha.n.y>-odchylka
     &&vysledek.plocha.n.z<odchylka&&vysledek.plocha.n.z>-odchylka);
  vysledek.body=body;
  *mn=vysledek;
  return TRUE;
}

int inicGraf(void)
{
  int gr_driver=DETECT,gmode,gerr;
  initgraph(&gr_driver,&gmode,BGI);
  if ( ( gerr=graphresult()) != grOk ){
    fprintf(stderr,"%s",grapherrormsg(gerr));
    exit (FALSE);
  }
  else
    setviewport(0,0,getmaxx(),getmaxy(),1);
  return TRUE;
}

char *hlasky[DOBRE]={"Spatny format souboru!",
      "V souboru je malo souradnic!",
      "Mnohouhelnik musi mit alespon tri vrcholy!",
      "Zadane souradnice nedefinuji konvexni rovinny mnohouhelnik."};

int nactiPlochu(FILE *data,mnohouhelnik *vysledek)
{
  char p_ret[RETEZEC];
  int j,i,k,bodu,znak;
  int sp_format=FALSE;
  float a,b,c;
  bod3d *point;

  i=0;
  bodu=0;
  while(!feof(data))
  {
    znak=getc(data);
    if(znak=='\n'||znak==EOF)
    {
      p_ret[i]='\0';
      k=sscanf(p_ret,"%d",&bodu);
      if(k==0||k==EOF)
      {
	i=0;
	continue;
      }
      if(k!=1 )
      {
	return FORMAT_SOUB;
      }
      else
	break;
    }
    if(znak<'0'||znak>'9')
      p_ret[i]=' ';
    else
      p_ret[i]=znak;
    i++;
    if(i==RETEZEC)
      return FORMAT_SOUB;
  }

  if(feof(data))
  {
    return MALO_SOUR;
  }
  if((point=(bod3d*)calloc(bodu,sizeof(bod3d)))==NULL){
    fprintf(stderr,"Malo pameti pro beh programu!\n");
    exit(0);
  }

  for(j=0;j<bodu;)
  {
    i=0;

    while(!feof(data))
    {
      znak=getc(data);
      if(znak=='\n'||feof(data))
      {
	p_ret[i]='\0';
	k=sscanf(p_ret,"%f %f %f",&a,&b,&c);
	point[j].x=a;
	point[j].y=b;
	point[j].z=c;
	if(k==0||k==EOF)
	{
	  i=0;
	  continue;
	}
	if(k!=3 )
	{
	  sp_format=TRUE;
	}
	else
	  break;
      }
      if((znak<'0'||znak>'9') && znak != '.' && znak !='e'
		 && znak !='E' && znak !='\0' && znak !='-')
	p_ret[i]=' ';
      else
	p_ret[i]=znak;
      i++;
      if(i==RETEZEC)
	return FORMAT_SOUB;

    }
    if(feof(data)&&j<bodu-1)
    {
      free(point);
      return MALO_SOUR;
    }
    j++;
  }
  if(bodu<BODU_PLOCHY)
  {
    free(point);
    return NEDEF_RNA;
  }
  if(sp_format==TRUE)
    return FORMAT_SOUB;

  if(kontrola(point,bodu)==FALSE){
    return PROT_HRAN;
  }
  if(utvorMnohouhelnik(point,bodu,vysledek)==FALSE)
    return NENI_VIDET;
  return DOBRE;
}


int nactiSoubor(FILE *data,mnohouhelnik *polemnoh){
  int i,j,k,pocet_chyb,sp_format;

  i=0;
  j=1;
  clrscr();
  pocet_chyb=0;
  sp_format=0;
  while(!feof(data)&&i<MAX_MNOH){
    if((k=nactiPlochu(data,&polemnoh[i]))!=DOBRE){
      if((k==NEDEF_RNA && !feof(data))||k==PROT_HRAN||k==FORMAT_SOUB||
	    k==MALO_SOUR){
	pocet_chyb++;
	printf("%d. mnohouhelnik byl spatne zadan!\n",j);
	printf("(%s)\n\n",hlasky[k]);
	if(k==FORMAT_SOUB){
	  sp_format++;
	  if(sp_format>3)
	    return -i;
	}
      }
    }
    else{
      if(polemnoh[i].plocha.n.z>odchylka||polemnoh[i].plocha.n.z<-odchylka){
	polemnoh[i].barva=i+1;
	i++;
      }
    }
    j++;
  }
  if(pocet_chyb>0)
    return -i;
  else
    return i;
}

void urciMaxima(mnohouhelnik *mn,int pocetmnoh,float *max,float *may,
    float *mix, float *miy, float *miz){
  *mix=mn[0].mix;
  *max=mn[0].max;
  *miy=mn[0].miy;
  *may=mn[0].may;
  *miz=mn[0].miz;

  for(int i=0;i<pocetmnoh;i++){
    if(mn[i].mix<*mix)
      *mix=mn[i].mix;
    if(mn[i].max>*max)
      *max=mn[i].max;
    if(mn[i].miy<*miy)
      *miy=mn[i].miy;
    if(mn[i].may>*may)
      *may=mn[i].may;
    if(mn[i].miz<*miz)
      *miz=mn[i].miz;
  }
}

void urciKoef(mnohouhelnik *mn,int pocetmnoh,float maxx,float maxy, float *krok,
  float *Ox,float *Oy, float *miz){
  float max,may,mix,miy;
  float krokx,kroky,Sx,Sy,delkax,delkay;
  urciMaxima(mn,pocetmnoh,&max,&may,&mix,&miy,miz);
  delkax=max-mix;
  Sx=mix+delkax/2;
  delkay=may-miy;
  Sy=may-delkay/2;
  krokx=delkax*OKRAJE/maxx;
  kroky=delkay*OKRAJE/maxy;
  *krok=(krokx>kroky)?krokx:kroky;
  *Ox=Sx-(maxx/2)*(*krok);
  *Oy=Sy+(maxy/2)*(*krok);
}


int zbuf(mnohouhelnik *mn,int pocetmnoh,long int maxx,long int maxy){

  int i,j,k;
  pol_buf *buffer;
  float krok,Ox,Oy,minz,y;

  urciKoef(mn,pocetmnoh,maxx,maxy,&krok,&Ox,&Oy,&minz);

  if((buffer=(pol_buf *)calloc(maxx,sizeof(pol_buf)))==NULL){
    fprintf(stderr,"Nedostatek pameti pro vypocet!\n");
    printf("Stiskni cokoliv pro navrat do menu.\n");
    while(!kbhit());
    if(getch()=='\0')
      getch();
    return FALSE;
  }
  y=Oy;

  for(i=0;i<maxy;i++){
    for(j=0;j<maxx;j++){
      buffer[j].barva=0;
      buffer[j].sour_z=minz-1;
    }
    for(k=0;k<pocetmnoh;k++){
      if(mn[k].may>=y&&mn[k].miy<=y){
	vyplnBuffer(buffer,mn[k],y,Ox,krok);
      }
    }
    for(j=0;j<maxx;j++){
      putpixel(j,i,buffer[j].barva);
    }
    y-=krok;
  }
  free(buffer);
  return TRUE;
}


void vid2(mnohouhelnik *mn,int pocetmnoh,long int maxx,long int maxy){

  int i,j,k,barva;
  bod3d aktualni;
  float krok,Ox,Oy,minz,z;

  urciKoef(mn,pocetmnoh,maxx,maxy,&krok,&Ox,&Oy,&minz);
  aktualni.x=Ox;
  aktualni.y=Oy;

  for(i=0;i<maxy;i++){
    for(j=0;j<maxx;j++){
      barva=0;
      z=minz-1;
      for(k=0;k<pocetmnoh;k++){
	if(aktualni.y<=mn[k].may&&aktualni.y>=mn[k].miy){
	if(patriPlose(mn[k],aktualni)==TRUE){
	  aktualni.z=((-mn[k].plocha.n.x*aktualni.x)-(mn[k].plocha.n.y*aktualni.y)
	      -mn[k].plocha.d)/mn[k].plocha.n.z;
	  if(aktualni.z>z){
	    z=aktualni.z;
	    barva=mn[k].barva;
	  }
	}
	}
      }
      aktualni.x+=krok;
      putpixel(j,i,barva);
    }
    aktualni.y-=krok;
    aktualni.x=Ox;
  }
}


int novySoubor(FILE **data){
  char retezec[RETEZEC];
  int i,znak;
  FILE *novy;

  while(TRUE){
    i=0;
    clrscr();
    printf("Zadej jmeno souboru se souradnicemi (<*> - zpet do menu):\n");
    retezec[i]=getchar();
    while(retezec[i]!='\n'&&i<RETEZEC-1){
      if(retezec[i]=='*')
	return FALSE;
      i++;
      retezec[i]=getchar();
    }
    retezec[i]='\0';
    if((novy=fopen(retezec,"r"))==NULL){
      clrscr();
      printf("Soubor \"%s\" nelze otevrit.\n",retezec);
      printf("<Q> - navrat do menu\n");
      printf("<ENTER> - zadat nove jmeno\n");
      while(TRUE){
	while(!kbhit());
	if((znak=getch())=='\0')
	  znak=getch();
	else{
	  if(znak==13)
	    break;
	  if(znak=='q' || znak=='Q'){
	    return FALSE;
	  }
	}
      }
    }
    else{
      fclose(*data);
      *data=novy;
      return TRUE;
    }
  }
}


int napoveda(){
  char znak;
  int stranka=1;

  while(TRUE){
  clrscr();
  switch(stranka){
    case 1:
      puts("**********************************VIDITELNOST"
	"**********************************\n");
      puts("Program slouzi k promitani rovinnych mnohouhelniku v prostoru do roviny 0xy.");
      puts("K zobrazeni je pouzita perspektiva s pozorovatelem v nekonecnu kladne osy z.");
      puts("Pri tomto promitani jsou vykresleny jen viditelne casti mnohouhelniku.");
      puts("\n\nOvladani\n");
      puts("V menu lze vybrat jednu z polozek pomoci klavesy uvedene u dane volby.");
      puts("<S> - je pozadovano jmeno souboru. Z tohoto souboru jsou nacteny souradnice.");
      puts("<Z>, <V> - pokud byl zadan soubor se souradnicemi, jsou vykresleny mnoho-");
      puts("uhelniky pomoci prislusneho algoritmu (viz. algoritmy vypoctu).");
      puts("Po vykresleni se stiskem libovolne klavesy lze vratit do menu.");
      puts("<C> - pokud jiz bylo provedeno vykresleni, jsou zobrazeny casy potrebne pro");
      puts("vypocet a nasledne vykresleni.");
      puts("\n\n\n<A> - algoritmy vypoctu");
      puts("<M> - mnohouhelniky (omezeni, zpusob zadavani)");
      puts("<ENTER> - konec napovedy");
      break;
    case 2:
      puts("ALGORITMY VYPOCTU\n");
      puts("Pro vypocet lze pouzit jeden ze dvou algoritmu:");
      puts("Modifikace z-bufferu: modifikace algoritmu spociva vjeho y-nasobnem pouziti");
      puts("      (kde y je pocet radku obrazovky). Pro kazdy radek je pouzit buffer,");
      puts("      do ktereho jsou ukladany hodnoty z-tove souradnice a barva. Pro");
      puts("      vsechny mnohouhelniky jsou urcene pixely radku, do kterych se mnoho-");
      puts("      uhelniky zobrazi a podle hodnoty z-tove souradnice je pripadne");
      puts("      modifikavana barva daneho pixelu. Nasledne je radek vykreslen");
      puts("      a algoritmus se opakuje.");
      puts("      Nevyhoda algoritmu: pro buffer je potreba alokovat x*6 bytu pameti ");
      puts("      (kde x je pocet pixelu na radku).");
      puts("Vypocet po jednotlivych pixelech: postupne jsou pro kazdy pixel prochazeny");
      puts("      vsechny mnohouhelniky. Pokud se ma mnohouhelnik promitnout do daneho");
      puts("      pixelu je jeho z-souradnice porovnana se z-souradnicemi ostatnich");
      puts("      takovychto mnohouhelniku. Pixel je nasledne vykreslen barvou prislusnou");
      puts("      mnohouhelniku s nejvetsi z-tovou souradnici.");
      puts("      Nevohoda algoritmu: algoritmus je znacne pomalejsi nez drive popsany.");
      puts("\n<N> - popis programu a ovladani");
      puts("<M> - mnohouhelniky (omezeni, zpusob zadavani)");
      puts("<ENTER> - konec napovedy");
      break;
    case 3:
      puts("MNOHOUHELNIKY\n");
      puts("Program zobrazuje maximalne 15 mnohouhelniku. Ty jsou zadavany pomoci souboru.");
      puts("Pocet vrcholu mnohouhelnika neni omezen, ale vsechny musi lezet v jedne rovi-");
      puts("ne. Mnohouhelnik musi byt konvexni (vsechny vnitrni uhly musi byt ostre).");
      puts("Zobrazeny nemohou byt mnohouhelniky lezici v rovine kolme na rovinu 0xy,");
      puts("protoze nani definovana tloustka roviny. Kazdemu mnohouhelniku je prirazena");
      puts("barva, kterou je vykreslen.");
      puts("\n\nFormat souboru\n");
      puts("Soubor se souradnicemi musi byt textovy. Privni radek obsahuje pocet vrcholu");
      puts("prvniho mnohouhelnika. Na dalsich radcich jsou potom souradnice bodu. Na");
      puts("kazdem radku jsou ulozeny vsechny 3 souradnice daneho vrcholu. Vrcholy ulozene");
      puts("v radcich pod sebou musi mit spolecnou hranu. Na dalsim radku po vsech sourad-");
      puts("nicich vrcholu je pocet vrcholu dalsiho mnohouhelnika. Toto se opakuje pro vse-");
      puts("chny mnohouhelniky.");
      puts("\n\n<N> - popis programu a ovladani");
      puts("<A> - algoritmy vypoctu");
      puts("<ENTER> - konec napovedy");
      break;
  }
    while(!kbhit());
    if((znak=getch())=='\0')
      getch();
    else
      switch(znak){
	  case 'A':
	  case 'a':stranka=2;
		 break;
	  case 'M':
	  case 'm':stranka=3;
		 break;
	  case 'N':
	  case 'n':stranka=1;
		  break;
	  case 13:return TRUE;
      }
  }
}



void casy(clock_t start1,clock_t end1,clock_t start2,clock_t end2){
  clrscr();
  if(start1!=0)
    printf("Cas potrebny pro vypocet modifikovanym algoritmem"
	" z-buffer: %8.6f s\n\n",(end1-start1)/(double) CLK_TCK);
  else
    printf("Vypocet modifikovanym algoritmem z-buffer dosud nebyl proveden!\n\n");
  if(start2!=0)
    printf("Cas potrebny pro vypocet po jednotlivych pixelech: "
	"%8.6f s\n\n\n\n",(end2-start2)/(double) CLK_TCK);
  else
    printf("Vypocet po jednotlivych pixelech dosud nebyl proveden!\n\n\n\n");
  printf("Stiskni cokoliv pro navrat do menu.");
  while(!kbhit());
  if(getch()=='\0')
    getch();
}

void menu(){
  int znak,j,maxx,maxy;
  int poc=0;
  int poc2=0;
  int novy_soub=FALSE;
  int nacteny_data=FALSE;
  clock_t start1=0;
  clock_t end1=0;
  clock_t start2=0;
  clock_t end2=0;
  FILE *s;
  mnohouhelnik mn[15];
  mnohouhelnik mn2[15];

  while(TRUE){
    clrscr();
    printf("**********************************VIDITELNOST"
    "**********************************\n\n");
    printf("<S> - nacist novy soubor\n<C> - porovnani casu vypoctu\n"
    "<Z> - zobrazit mnohouhelniky pomoci modifikace algoritmu z-buffer\n");
    printf("<V> - zobrazit mnohouhelniky pomoci vypoctu pro kazdy pixel\n"
      "<N> - napoveda\n<K> - konec programu\n\n");
    while(!kbhit());
    znak=getch();
    if(znak=='\0')
      getch();
    else{
      switch(znak){
	case 'S':
	case 's':novy_soub=novySoubor(&s);
		 if(novy_soub==TRUE){
		   poc2=poc;
		   poc=nactiSoubor(s,mn2);
		   if(poc<0){
		     poc=-poc;
		     printf("Spravne nactenych mnohouhelniku: %d\n",poc);
		     printf("Stiskni cokoliv.\n");
		     while(!kbhit());
		     if((znak=getch())=='\0')
		       znak=getch();
		   }
		   if(poc>0){
		     nacteny_data=TRUE;
		     for(j=0;j<poc2;j++){
		       free(mn[j].pol);
		       free(mn[j].body);
		     }
		     for(j=0;j<poc;j++){
		       mn[j]=mn2[j];
		     }
		     start1=end1=start2=end2=0;
		   }
		   else{
		     poc=poc2;
		     printf("V danem souboru neni zadny spravne zadany"
		     " zobrazitelny mnohouhelnik.\n");
		     printf("Stiskni cokoliv.\n");
		     while(!kbhit());
		     if((znak=getch())=='\0')
		       znak=getch();
		   }
		 }
		 break;
	case 'Z':
	case 'z':if(nacteny_data==TRUE){
		   inicGraf();
		   maxx=getmaxx();
		   maxy=getmaxy();
		   start1=clock();
		   zbuf(mn,poc,maxx,maxy);
		   end1=clock();
		   while(!kbhit());
		   if(getch()=='\0')
		     getch();
		   closegraph();
		   clrscr();
		   continue;
		 }
		 else{
		   clrscr();
		   printf("Nebyl zadan zadny soubor se souradnicemi.\n");
		   printf("Stiskni cokoliv.\n");
		   while(!kbhit());
		   if((znak=getch())=='\0')
		     znak=getch();
		 }
		 break;
	case 'V':
	case 'v':if(nacteny_data==TRUE){
		   inicGraf();
		   maxx=getmaxx();
		   maxy=getmaxy();
		   start2=clock();
		   vid2(mn,poc,maxx,maxy);
		   end2=clock();
		   while(!kbhit());
		   if(getch()=='\0')
		     getch();
		   closegraph();
		   clrscr();
		   continue;
		 }
		 else{
		   clrscr();
		   printf("Nebyl zadan zadny soubor se souradnicemi.\n");
		   printf("Stiskni cokoliv.\n");
		   while(!kbhit());
		   if((znak=getch())=='\0')
		     znak=getch();
		 }
		 break;
	case 'C':
	case 'c':casy(start1,end1,start2,end2);
		 break;
	case 'N':
	case 'n':napoveda();
		 break;
	case 'K':
	case 'k':exit(0);
		 break;

      }
    }
  }
}

int main(){
  menu();
  return 0;
}
